@@ -1,6 +1,6 @@
 /* client_keyexchange.c
  *
- * $Id: client_keyexchange.c,v 1.44 2000/05/28 21:09:44 nisse Exp $ */
+ * $Id: client_keyexchange.c,v 1.47 2000/06/01 17:01:54 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -55,7 +55,7 @@
        (dh struct dh_instance)
        (hostkey_algorithm . UINT32)
        (verifier object lookup_verifier)
-       (install object install_keys)))
+       (algorithms object object_list)))
 */
     
 static void
@@ -110,7 +110,7 @@ do_handle_dh_reply(struct packet_handler *c,
   hash = closure->dh.exchange_hash;
   closure->dh.exchange_hash = NULL; /* For gc */
 
-  keyexchange_finish(connection, closure->install,
+  keyexchange_finish(connection, closure->algorithms,
 		     closure->dh.method->H,
 		     hash,
 		     ssh_format("%n", closure->dh.K));
@@ -137,7 +137,7 @@ do_init_client_dh(struct keyexchange_algorithm *c,
   dh->verifier = verifier;
   dh->hostkey_algorithm = hostkey_algorithm_atom;
 
-  dh->install = make_install_new_keys(0, algorithms);
+  dh->algorithms = algorithms;
   
   /* Send client's message */
   C_WRITE(connection, dh_make_client_msg(&dh->dh));
@@ -171,7 +171,7 @@ make_dh_client(struct dh_method *dh)
        (dh struct dh_instance)
        (name string)
        ;; (salt string)
-       (install object install_keys)))
+       (algorithms object object_list)))
 */
 
 /* GABA:
@@ -201,7 +201,9 @@ do_srp_client_proof_handler(struct packet_handler *s,
       hash = self->srp->dh.exchange_hash;
       self->srp->dh.exchange_hash = NULL; /* For gc */
 
-      keyexchange_finish(connection, self->srp->install,
+      connection->flags |= CONNECTION_SRP;
+      
+      keyexchange_finish(connection, self->srp->algorithms,
 			 self->srp->dh.method->H,
 			 hash,
 			 ssh_format("%n", self->srp->dh.K));
@@ -234,6 +236,8 @@ do_handle_srp_reply(struct packet_handler *s,
   CAST(srp_client_handler, self, s);
   struct lsh_string *salt = srp_process_reply_msg(&self->srp->dh, packet);
   struct lsh_string *passwd;
+  struct lsh_string *response;
+  
   mpz_t x;
   
   connection->dispatch[SSH_MSG_KEXSRP_REPLY] = connection->fail;
@@ -261,11 +265,18 @@ do_handle_srp_reply(struct packet_handler *s,
 
   lsh_string_free(salt);
   lsh_string_free(passwd);
-  
-  C_WRITE(connection, srp_make_client_proof(&self->srp->dh, x));
+
+  response = srp_make_client_proof(&self->srp->dh, x);
   mpz_clear(x);
 
-  connection->dispatch[SSH_MSG_KEXSRP_PROOF] = make_srp_client_proof_handler(self->srp);
+  if (!response)
+    PROTOCOL_ERROR(connection->e,
+		   "SRP failure: Invalid public value from server.");
+  
+  C_WRITE(connection, response);
+  
+  connection->dispatch[SSH_MSG_KEXSRP_PROOF]
+    = make_srp_client_proof_handler(self->srp);
 }
 
 static struct packet_handler *
@@ -301,7 +312,7 @@ do_init_client_srp(struct keyexchange_algorithm *s,
   /* Initialize */
   init_dh_instance(self->dh, &srp->dh, connection);
 
-  srp->install = make_install_new_keys(0, algorithms);
+  srp->algorithms = algorithms;
   srp->name = self->name;
   
   /* Send client's message */
@@ -318,6 +329,9 @@ make_srp_client(struct dh_method *dh, struct lsh_string *name)
 {
   NEW(srp_client_exchange, self);
 
+  assert(dh->G->add);
+  assert(dh->G->subtract);
+
   self->super.init = do_init_client_srp;
   self->dh = dh;
   self->name = name;
