@@ -312,8 +312,6 @@ do_userauth_banner(struct packet_handler *self UNUSED,
   UINT32 language_length;
   const UINT8 *language;
   
-  CHECK_TYPE(packet_handler, self);
-
   simple_buffer_init(&buffer, packet->length, packet->data);
 
   if (parse_uint8(&buffer, &msg_number)
@@ -654,8 +652,8 @@ do_userauth_pk_ok(struct packet_handler *s,
 						ATOM_SSH_USERAUTH,
 						key->type,
 						key->public);
-#endif  
 	  else
+#endif /* DATAFELLOWS_WORKAROUNDS */ 
 	    request = format_userauth_publickey(local_to_utf8(self->state->userauth->username, 0),
 						self->state->userauth->service_name,
 						key->type,
