@@ -1,6 +1,6 @@
 /* server_userauth.h
  *
- * $Id: server_userauth.h,v 1.11 2000/05/27 13:06:07 nisse Exp $
+ * $Id: server_userauth.h,v 1.13 2000/06/01 13:18:09 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -90,13 +90,6 @@ format_userauth_failure(struct int_list *methods,
 struct lsh_string *
 format_userauth_success(void);
 
-struct packet_handler *
-make_userauth_handler(struct alist *methods,
-                      struct alist *services,
-                      struct command_continuation *c,
-                      struct exception_handler *e);
-
-
 /* authentication methods */
 
 struct userauth *
